#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for amazon s3."""
from __future__ import with_statement

import shutil
import threading

from iobase.cmdexec import (CmdExec, NULL)
from iobase.exception import ExecError
import libpal.palcmd as palcmd
from pal_info import (ReqTypes, Results)

import pal_config as config

from boto.s3.connection import S3Connection, S3ResponseError

SHARENAME_NAME = "sharename"
RESULT_NAME = "result"
ACCESSKEY_NAME = "accesskey"
SECRETKEY_NAME = "secretkey"
BUCKET_NAME = "bucket"
PROXY_SERVER_PATH = "proxy/serverlist/server"
PROXYHOST_PATH = PROXY_SERVER_PATH + "/host"
PROXYPORT_PATH = PROXY_SERVER_PATH + "/port"
ENDPOINT_NAME = "endpoint"
PORT_NAME = "port"
HTTPMETHOD_NAME = "httpmethod"

SEND_DOWNLOAD_SIGNAL_ARGS = [config.NASDSYNC_INITD, "reload"]

DOWNLOAD_FAILED_MSG = "Download Failed!!"
CHECK_FAILED_MSG = "Check Failed!!"

SUCCESS_RESULT = "Success"
FAILED_RESULT = "Failed"
TIMEOUT_RESULT = "TimeOut"

ACCESS_TEST_NAME = "access_test.I-O_DATA.access_test"

HANDLEABLE_S3_ERROR_LIST = ("InvalidAccessKeyId",
                            "SignatureDoesNotMatch",
                            "NoSuchBucket")

NO_PROBLEM_ERROR = "NoSuchKey"

CHECK_TIMEOUT_SEC = 20

chk_result = FAILED_RESULT


def _exec_check(bucket):
    global chk_result

    try:
        bucket.get_all_keys(prefix=ACCESS_TEST_NAME)
        chk_result = SUCCESS_RESULT

    except S3ResponseError, ins:
        if ins.code in HANDLEABLE_S3_ERROR_LIST:
            chk_result = ins.code
        if ins.code == NO_PROBLEM_ERROR:
            chk_result = SUCCESS_RESULT

    except:
        pass


class PalAmazonS3(palcmd.PalCmd):
    """PAL AmazonS3 command implementation class."""

    def __make_download_share_file(self, share_name):

        with open(config.AMAZONS3_DOWNLOAD_TARGET_TEMP, "wb") as file:
            file.write(share_name)
        shutil.move(config.AMAZONS3_DOWNLOAD_TARGET_TEMP,
            config.AMAZONS3_DOWNLOAD_TARGET)

    def __download_handler(self, entry):
        """Handle download request."""

        try:
            share_name = entry.get().get_node(SHARENAME_NAME).get_text()
            self.__make_download_share_file(share_name)

            CmdExec(SEND_DOWNLOAD_SIGNAL_ARGS).execute(
                stdout=NULL, stderr=NULL)
            entry.set_result(Results.OK)

        except ExecError:
            self.exception(DOWNLOAD_FAILED_MSG)
            entry.set_result(Results.DOWNLOAD_ERROR)

        return entry

    def __check_handler(self, entry):
        """Handle check request."""

        global chk_result

        try:
            entry.set_result(Results.OK)

            body = entry.get()

            access_key = body.get_value(ACCESSKEY_NAME)
            secret_key = body.get_value(SECRETKEY_NAME)
            bucket_name = body.get_value(BUCKET_NAME)
            proxy_host = body.get_value(PROXYHOST_PATH, None)
            proxy_port = body.get_value(PROXYPORT_PATH, None)
            host = body.get_value(ENDPOINT_NAME)
            port = body.get_value(PORT_NAME, "")

            is_secure = {"https": True, "http": False}.get(
                body.get_value(HTTPMETHOD_NAME, "https"))

            if not host:
                host = S3Connection.DefaultHost

            if port == "":
                port = None
            else:
                port = int(port)

            conn = S3Connection(access_key,
                                secret_key,
                                proxy=proxy_host,
                                proxy_port=proxy_port,
                                host=host,
                                is_secure=is_secure,
                                port=port)

            bucket = conn.get_bucket(bucket_name, False)

            t = threading.Thread(target=_exec_check, args=(bucket,))
            t.setDaemon(True)
            t.start()
            t.join(CHECK_TIMEOUT_SEC)
            if t.isAlive():
                chk_result = TIMEOUT_RESULT

        except:
            chk_result = FAILED_RESULT
            entry.set_result(Results.CHECK_ERROR)

        finally:
            end_result = chk_result

            if end_result != SUCCESS_RESULT:
                self.exception(CHECK_FAILED_MSG)

            entry.get().append_child(RESULT_NAME, end_result)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.DOWNLOAD, self.__download_handler)
        self.set_handler(ReqTypes.CHECK, self.__check_handler)

#
# Main
#

if __name__ == "__main__":
    PalAmazonS3(option=palcmd.PalCmdOption()).handle_request()
