#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for processing the events and event handler."""


from __future__ import with_statement

import os
import errno
import glob

import iobase.xmldata as xmldata
import iobase.fileut as fileut
import iobase.cmdexec as cmdexec
import ionas.request as request
from iobase.exception import (InvalidArgError,)

import libpal.palcmd as palcmd
import pal_config as config
from pal_info import (ReqTypes, Results)


EVENTS_NAME = "events"
EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR_NAME = "category"
ACTION_ATTR_NAME = "action"
PREVIOUS_ATTR_NAME = "previous"
PRIORITY_ATTR_NAME = "priority"
VOLATILE_ATTR_NAME = "volatile"
COUNT_ATTR_NAME = "count"

EVENT_NAME_SEP = "-"
PREVIOUS_MARK = "P"
ALL_CATEGORY = "all"
DEFAULT_PRIORITY = "50"
PRIORITY_PADDING = "0"

ANY_HANDLER = "*"
ANY_PRIORITY = "[0-9][0-9]"
HANDLER_DIR_LIST = (config.EVENTS_TEMP_HANDLER_DIR, config.EVENTS_HANDLER_DIR)


class PalEvents(palcmd.PalCmd):
    """Pal conffile implementation class."""

    def __check_body_node(self, node):
        """Check if entry's body node is valid."""

        if node.get_node_name() != EVENTS_NAME:
            raise InvalidArgError(node.get_node_name())

    def __get_registered_name(self, handler_path, eventinfo):
        """Return the registered name based on "eventinfo" for
           "handler_path".
        """

        if handler_path == ANY_HANDLER:
            priority = ANY_PRIORITY
        else:
            priority = eventinfo.get_attr_value(PRIORITY_ATTR_NAME,
                                                DEFAULT_PRIORITY)
            if len(priority) == 1:
                priority = PRIORITY_PADDING + priority

        if eventinfo.is_set_to_true(PREVIOUS_ATTR_NAME):
            previous_mark = PREVIOUS_MARK
        else:
            previous_mark = ""

        return priority + previous_mark + EVENT_NAME_SEP + \
                    os.path.basename(handler_path)

    def __get_registered_sub_path(self, handler_path, eventinfo, category=""):
        """Return the registered handler sub path based on "eventinfo" for
           "handler_path" under the handler or counter directory.
        """

        if not category:
            category = eventinfo.get_attr_value(CATEGORY_ATTR_NAME,
                                ALL_CATEGORY)

        return os.path.join(category,
                        eventinfo.get_attr_value(ACTION_ATTR_NAME),
                        self.__get_registered_name(handler_path, eventinfo))

    def __get_registered_path(self, handler_path, eventinfo):
        """Return the registered handler path based on "eventinfo" for
           "handler_path".
        """

        if eventinfo.is_set_to_true(VOLATILE_ATTR_NAME):
            handler_dir = config.EVENTS_TEMP_HANDLER_DIR
        else:
            handler_dir = config.EVENTS_HANDLER_DIR

        return os.path.join(handler_dir,
                    self.__get_registered_sub_path(handler_path, eventinfo))

    def __get_counter_path(self, handler_path, eventinfo):
        """Return the counter path for "handler_path"."""

        if not eventinfo.get_attr_value(COUNT_ATTR_NAME, ""):
            return ""

        if eventinfo.is_set_to_true(VOLATILE_ATTR_NAME):
            counter_dir = config.EVENTS_TEMP_COUNTER_DIR
        else:
            counter_dir = config.EVENTS_COUNTER_DIR

        return os.path.join(counter_dir,
                    self.__get_registered_sub_path(handler_path, eventinfo))

    def __prepare_dir(self, file_path):
        """Create directory recursively for "file_path"."""

        if file_path and not os.path.exists(os.path.dirname(file_path)):
            os.makedirs(os.path.dirname(file_path))

    def __get_original_path(self, path):
        """Return the absolute path to which "path" points if "path" is
           symbolic link. Otherwise, return the absolute path of "path".
        """

        if os.path.islink(path):
            return os.path.abspath(
                os.path.join(os.path.dirname(path), os.readlink(path)))

        return os.path.abspath(path)

    def __register_handler(self, entry):
        """Handler for "register" request."""

        self.__check_body_node(entry.get())

        handler_path = entry.get_file()
        eventinfo = entry.get().get_node(EVENTINFO_NAME)
        registered_path = self.__get_registered_path(handler_path, eventinfo)
        counter_path = self.__get_counter_path(handler_path, eventinfo)

        try:
            self.__prepare_dir(registered_path)
            self.__prepare_dir(counter_path)

            try:
                if os.path.exists(registered_path):
                    os.remove(registered_path)
            except:
                self.exception("Register : delete error")
                entry.set_result(Results.DELETE_ERROR)
                return entry

            os.symlink(handler_path, registered_path)
            if counter_path:
                count = int(eventinfo.get_attr_value(COUNT_ATTR_NAME, ""))
                with fileut.FLock(config.LOCK_DIR,
                        os.path.basename(counter_path), exclusive=True):
                    with open(counter_path, "w") as counter:
                        counter.write(str(count))
            entry.set_result(Results.OK)

        except ValueError, err:
            self.exception(err.message)
            raise InvalidArgError("ValueError")
        except:
            self.exception("Register : error")
            entry.set_result(Results.CREATE_ERROR)

        return entry

    def __unregister_handler(self, entry):
        """Handler for "unregister" request."""

        self.__check_body_node(entry.get())

        eventinfo = entry.get().get_node(EVENTINFO_NAME)
        registered_path = self.__get_registered_path(entry.get_file(),
                                eventinfo)
        counter_path = self.__get_counter_path(entry.get_file(), eventinfo)

        try:
            if os.path.exists(registered_path):
                os.remove(registered_path)

            if counter_path and os.path.exists(counter_path):
                os.remove(counter_path)
            entry.set_result(Results.OK)
        except:
            self.exception("Unregister : delete error")
            entry.set_result(Results.DELETE_ERROR)

        return entry

    def __get_registered_handler_list(self, eventinfo):
        """Find the registerd handler for "eventinfo" and retern
           the list to it.
        """

        category_handler_glob = self.__get_registered_sub_path(ANY_HANDLER,
                                eventinfo)
        all_handler_glob = self.__get_registered_sub_path(ANY_HANDLER,
                                eventinfo, ALL_CATEGORY)

        handler_list = []
        for events_dir in HANDLER_DIR_LIST:
            for handler_glob in (category_handler_glob, all_handler_glob):
                for handler in \
                        glob.glob(os.path.join(events_dir, handler_glob)):

                    if not handler.endswith(".pyc"):
                        handler_list.append(handler)

        return sorted(handler_list, key=lambda path: os.path.basename(path))

    def __to_counter_path(self, handler):
        """Return the counter path for "handler"."""

        if handler.startswith(config.EVENTS_TEMP_HANDLER_DIR):
            return handler.replace(config.EVENTS_TEMP_HANDLER_DIR,
                                config.EVENTS_TEMP_COUNTER_DIR, 1)
        if handler.startswith(config.EVENTS_HANDLER_DIR):
            return handler.replace(config.EVENTS_HANDLER_DIR,
                                config.EVENTS_COUNTER_DIR, 1)
        return ""

    def __execute_handler(self, entry, handler):
        """Execute handler, update counter if it exists and
           unregister handler if counter == 0."""

        self.debug("Call handler: " + handler)
        result_entry = self.send_entry(entry, handler)
        counter_path = self.__to_counter_path(handler)

        if not counter_path or not os.path.exists(counter_path):
            return result_entry

        with fileut.FLock(config.LOCK_DIR, os.path.basename(counter_path),
                exclusive=True):
            with open(counter_path) as counter:
                count = int(counter.read().strip())

            if count > 0:
                count -= 1
            if count > 0:
                with open(counter_path, "w") as counter:
                    counter.write(str(count))
            else:
                os.remove(handler)
                os.remove(counter_path)
        return result_entry

    def __event_handler(self, entry):
        """Handler for "event" request."""

        self.__check_body_node(entry.get())

        handler = "not found"
        error_count = 0
        try:
            for handler in self.__get_registered_handler_list(
                                entry.get().get_node(EVENTINFO_NAME)):

                try:
                    result_entry = self.__execute_handler(entry, handler)
                    if result_entry.get_result() != Results.OK:
                        error_count += 1
                        self.error("Handler %s result: %s" %
                                        (handler, result_entry.get_result()))
                        continue
                except:
                    error_count += 1
                    self.exception("Handler %s" % (handler,))
                    continue

            if error_count:
                entry.set_result(Results.EXECUTE_ERROR)
            else:
                entry.set_result(Results.OK)
        except:
            self.exception("Handler execution error: " + handler)
            entry.set_result(Results.EXECUTE_ERROR)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.REGISTER, self.__register_handler)
        self.set_handler(ReqTypes.UNREGISTER, self.__unregister_handler)
        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    PalEvents(option=palcmd.PalCmdOption()).handle_request()
