#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for GROUP configuration."""

import grp

from iobase.xmldata import XmlNode
from iobase.cmdexec import (CmdExec, NULL)
from iobase.exception import (ExecError, InvalidArgError)
import libpal.palcmd as palcmd
from libpal.util import RemountRoot
from pal_info import (ReqTypes, Results)

import pal_config as config


GROUPS_NAME = "groups"
GROUP_NAME = "group"
NAME_NAME = "name"
GID_NAME = "gid"
USERLIST_NAME = "userlist"

INVALID_ROOT_TAG_MSG = "Invalid tag name!!"
ADD_FAILED_MSG = "Fail adding group!!"
DELETE_FAILED_MSG = "Fail deleting group!!"
GROUPNAME_EMPTY_MSG = "Group name empty!!"


class PalGroups(palcmd.PalCmd):
    """PAL Groups command implementation class."""

    def __check_root_tag(self, entry):
        """Check root tag."""

        root_name = entry.get().get_node_name()

        if root_name != GROUPS_NAME:
            raise InvalidArgError(INVALID_ROOT_TAG_MSG)

    def __is_empty(self, val):
        if val in (None, "", [], (), {}):
            return True
        return False

    def __add_handler(self, entry):
        """Handle add request."""

        try:
            self.__check_root_tag(entry)

            group_node = entry.get().get_node(GROUP_NAME)

            # リクエストされたグループ情報取得
            name = group_node.get_value(NAME_NAME, None)
            if self.__is_empty(name):
                raise InvalidArgError(GROUPNAME_EMPTY_MSG)
            gid = group_node.get_value(GID_NAME, None)
            members = self.__get_members(group_node)

            # group作成
            cmd = [config.GROUPADD_CMD]
            if not self.__is_empty(gid):
                cmd = cmd + ['-g', gid]
            cmd = cmd + [name]
            with RemountRoot():
                CmdExec(cmd).execute(stdout=NULL, stderr=NULL)

            # グループにユーザーを追加
            with RemountRoot():
                CmdExec(['gpasswd', '-M', ",".join(members), name]).execute(
                        stdout=NULL, stderr=NULL)

            # 追加したグループ情報取得
            info = grp.getgrnam(name)
            gid = info.gr_gid
            mem = info.gr_mem

            # レスポンスエントリ作成
            group_node = entry.get().get_node(GROUP_NAME)
            group_node.clear()
            group_node.append_child(NAME_NAME, text=name)
            group_node.append_child(GID_NAME, text=str(gid))
            group_node.append_child(USERLIST_NAME)
            userlist_node = group_node.get_node(USERLIST_NAME)
            for m in mem:
                user_node = XmlNode('user')
                user_node.append_child('name', text=m)
                userlist_node.append_child(user_node)

            entry.set_result(Results.OK)

        except ExecError:
            self.exception(ADD_FAILED_MSG)
            entry.set_result(Results.ADD_ERROR)

        return entry

    def __get_members(self, group_node):

        return [n.get_value('name', '')
                for n
                in group_node.get_node_list('userlist/user')
                if n.get_value('name', '')]

    def __modify_handler(self, entry):
        """Handle add request."""

        try:
            self.__check_root_tag(entry)

            group_node = entry.get().get_node(GROUP_NAME)

            # リクエストされたグループ情報取得
            name = group_node.get_value(NAME_NAME, None)
            if self.__is_empty(name):
                raise InvalidArgError(GROUPNAME_EMPTY_MSG)
            gid = group_node.get_value(GID_NAME, None)
            members = self.__get_members(group_node)

            # group変更
            cmd = [config.GROUPMOD_CMD]
            if not self.__is_empty(gid):
                cmd = cmd + ['-g', gid]
            cmd = cmd + [name]
            with RemountRoot():
                CmdExec(cmd).execute(stdout=NULL, stderr=NULL)

            # グループメンバーを変更
            with RemountRoot():
                CmdExec(['gpasswd', '-M', ",".join(members), name]).execute(
                        stdout=NULL, stderr=NULL)

            # 変更したグループ情報取得
            info = grp.getgrnam(name)
            gid = info.gr_gid
            mem = info.gr_mem

            # レスポンスエントリ作成
            group_node = entry.get().get_node(GROUP_NAME)
            group_node.clear()
            group_node.append_child(NAME_NAME, text=name)
            group_node.append_child(GID_NAME, text=str(gid))
            group_node.append_child(USERLIST_NAME)
            userlist_node = group_node.get_node(USERLIST_NAME)
            for m in mem:
                user_node = XmlNode('user')
                user_node.append_child('name', text=m)
                userlist_node.append_child(user_node)

            entry.set_result(Results.OK)

        except ExecError:
            self.exception(ADD_FAILED_MSG)
            entry.set_result(Results.ADD_ERROR)

        return entry

    def __delete_handler(self, entry):
        """Handle delete request."""

        try:
            self.__check_root_tag(entry)

            group_node = entry.get().get_node(GROUP_NAME)
#            name = group_node.get_node(NAME_NAME).get_text()
            name = group_node.get_value(NAME_NAME, None)
            if self.__is_empty(name):
                raise InvalidArgError(GROUPNAME_EMPTY_MSG)
            cmd = [config.GROUPDEL_CMD, name]

            with RemountRoot():
                CmdExec(cmd).execute(stdout=NULL, stderr=NULL)
            entry.set_result(Results.OK)

        except ExecError:
            self.exception(DELETE_FAILED_MSG)
            entry.set_result(Results.DELETE_ERROR)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.MODIFY, self.__modify_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)

#
# Main
#

if __name__ == "__main__":
    PalGroups(option=palcmd.PalCmdOption()).handle_request()
