#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for buffer clear."""

import os
import re
import shutil
import urllib
from iobase.cmdexec import (CmdExec, NULL, PIPE)
from iobase.exception import (ExecError, NotFoundError)
import libpal.palcmd as palcmd
from pal_info import (ReqTypes, Results)

import pal_config as config


CANCEL_FAILED_MSG = "Fail canceling!!"
PRINTER_CONF_STR = \
            "<DefaultPrinter USB_printer>\n" + \
            "DeviceURI usb://%s/%s\n" + \
            "State Idle\n" + \
            "Accepting Yes\n" + \
            "JobSheets none none\n" + \
            "QuotaPeriod 0\n" + \
            "PageLimit 0\n" + \
            "KLimit 0\n" + \
            "Info USB printer\n" + \
            "</Printer>"

REAL_LP_PREFIX = "real."


class PalPrinter(palcmd.PalCmd):
    """PAL Printer command implementation class."""

    def __get_dev_name(self):
        for dev in config.USB_LP_DEV_LIST:
            real_dev = config.DEV_DIR + "/" + REAL_LP_PREFIX + dev
            if os.path.exists(real_dev):
                return real_dev

            if os.path.exists(config.DEV_DIR + "/" + dev):
                return dev
        return ""

    def __get_printer_dev_uri(self):
        udevadm_cmd = [config.UDEVADM_CMD, "info",
                "--attribute-walk", "--name=%s" % self.__get_dev_name()]
        try:
            proc = CmdExec(udevadm_cmd, stdout=PIPE, stderr=NULL).execute()

            (vendor, model) = ("", "")
            for line in proc.stdout:
                match = re.search(
                        "\{ieee1284_id\}==.*MFG:([^;]+);.*MDL:([^;]+)", line)
                if not match:
                    continue
                (vendor, model) = (match.group(1), match.group(2))
                break
        except:
            pass
        return (vendor, model)

    def __update_cups_conf(self):
        (vendor, model) = self.__get_printer_dev_uri()
        if vendor == "" or model == "":
            raise NotFoundError("Printer Device Not Found!")
        tmp_file_name = "printers.conf." + str(os.getpid())
        tmp_file_path = os.path.join(config.TEMP_DIR, tmp_file_name)
        tmp_file = open(tmp_file_path, "w")
        tmp_file.write(
            PRINTER_CONF_STR % (urllib.quote(vendor), urllib.quote(model)))
        tmp_file.close()
        shutil.move(tmp_file_path, config.PRINTER_CONF_FILE)
        return

    def __disable_handler(self, entry):
        """Handle disable request."""

        # There is no process.
        entry.set_result(Results.OK)
        return entry

    def __enable_handler(self, entry):
        """Handle enable request."""

        try:
            self.__update_cups_conf()
        except NotFoundError:
            entry.set_result(Results.DEVICE_NOT_FOUND)
            return entry
        entry.set_result(Results.OK)
        return entry

    def __clear_handler(self, entry):
        """Handle clear request."""

        try:
            cmd = CmdExec(config.PRINTER_CUPS_CANCEL_ARGS)
            cmd.execute(stdout=NULL, stderr=NULL)
            entry.set_result(Results.OK)
        except ExecError:
            self.exception(CANCEL_FAILED_MSG)
            entry.set_result(Results.CLEAR_ERROR)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.CLEAR, self.__clear_handler)
        self.set_handler(ReqTypes.ENABLE, self.__enable_handler)
        self.set_handler(ReqTypes.DISABLE, self.__disable_handler)

#
# Main
#

if __name__ == "__main__":
    PalPrinter(option=palcmd.PalCmdOption()).handle_request()
