#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for controlling the service."""


from __future__ import with_statement

import os
import errno
import glob
import time

import iobase.xmldata as xmldata
import iobase.fileut as fileut
import ionas.request as request
from iobase.cmdexec import (CmdExec, NULL)
from iobase.exception import (InvalidArgError,)

import libpal.palcmd as palcmd
import pal_config as config
from pal_info import (ReqTypes, Results)


SERVICECTRL_NAME = "servicectrl"
SERVICE_PATH = ".//service"
NAME_NAME = "name"
SERVICE_NAME_PATH = SERVICE_PATH + "/" + NAME_NAME
RUNNING_STATUS = "running"
STOP_STATUS = "stop"
SUSPEND_STATUS = "suspend"
STATUS_LIST = (RUNNING_STATUS, STOP_STATUS, SUSPEND_STATUS)
SERVICE_WAIT_TIME = 2

START_OPT = "start"
STOP_OPT = "stop"

TRUE_TEXT = xmldata.XmlConfNode.TRUE_TEXT
FALSE_TEXT = xmldata.XmlConfNode.FALSE_TEXT

SUSPEND_SERVICE_GLOB = os.path.join(config.SERVICE_DIR, SUSPEND_STATUS, "*")


class PalServiceCtrl(palcmd.PalCmd):
    """Pal Service Control command implementation class."""

    def __check_body_node(self, node):
        """Check if entry's body node is valid."""

        if node.get_node_name() != SERVICECTRL_NAME:
            raise InvalidArgError(node.get_node_name())

    def __get_service_status(self, name):
        """Return service status."""

        for status in STATUS_LIST:
            status_path = os.path.join(config.SERVICE_DIR, status, name)
            if os.path.exists(status_path):
                return status

        return STOP_STATUS

    def __query_handler(self, entry):
        """Handler for "query" request."""

        self.__check_body_node(entry.get())

        for service in entry.get().get_node_list(SERVICE_PATH):
            name = service.get_node(NAME_NAME).get_text()
            attrs = {
                    RUNNING_STATUS: FALSE_TEXT,
                    STOP_STATUS: FALSE_TEXT,
                    SUSPEND_STATUS: FALSE_TEXT,
                }

            status = self.__get_service_status(name)
            if status:
                attrs[status] = TRUE_TEXT

            service.set_attrs(attrs)

        entry.set_result(Results.OK)
        return entry

    def __change_service_status(self, name, new_status):
        """Change service status."""

        for status in STATUS_LIST:
            status_path = os.path.join(config.SERVICE_DIR, status, name)
            if os.path.exists(status_path):
                os.remove(status_path)

        status_dir = os.path.join(config.SERVICE_DIR, new_status)
        if not os.path.exists(status_dir):
            os.makedirs(status_dir)

        with open(os.path.join(status_dir, name), "w"):
            pass

    def __control_service(self, name, do_stop):
        """Start or Stop services."""

        if do_stop:
            opt = STOP_OPT
        else:
            opt = START_OPT
        cmd_args = [os.path.join(config.INITD_DIR, name), opt]
        CmdExec(cmd_args, stdin=NULL, stdout=NULL, stderr=NULL,
                        close_fds=True).execute()

    def __wait_for_service_start_ready(self, name):
        """Wait for service start ready."""

        try:
            status_path = os.path.join(config.SERVICE_DIR, STOP_STATUS, name)
            if not os.path.exists(status_path):
                return

            time_current = time.time()
            time_stopped = os.path.getmtime(status_path)
            if time_current - time_stopped < SERVICE_WAIT_TIME:
                time.sleep(SERVICE_WAIT_TIME)
        except:
            pass

    def __start_handler(self, entry):
        """Handler for "start" request."""

        self.__check_body_node(entry.get())

        error_count = 0
        for name_node in entry.get().get_node_list(SERVICE_NAME_PATH):
            try:
                name = name_node.get_text()
                if self.__get_service_status(name) == RUNNING_STATUS:
                    continue

                self.__wait_for_service_start_ready(name)
                self.__control_service(name, do_stop=False)
                self.__change_service_status(name, RUNNING_STATUS)
            except:
                self.exception("Start error for " + name)
                error_count += 1

        if error_count:
            entry.set_result(Results.START_ERROR)
        else:
            entry.set_result(Results.OK)

        return entry

    def __stop_handler(self, entry):
        """Handler for "stop" request."""

        self.__check_body_node(entry.get())

        error_count = 0
        for name_node in entry.get().get_node_list(SERVICE_NAME_PATH):
            try:
                name = name_node.get_text()
                try:
                    self.__control_service(name, do_stop=True)
                except:
                    # Retry one more.
                    time.sleep(SERVICE_WAIT_TIME)
                    self.__control_service(name, do_stop=True)
                    
                self.__change_service_status(name, STOP_STATUS)
            except:
                self.exception("Stop error for " + name)
                error_count += 1

        if error_count:
            entry.set_result(Results.STOP_ERROR)
        else:
            entry.set_result(Results.OK)

        return entry

    def __suspend_handler(self, entry):
        """Handler for "suspend" request."""

        self.__check_body_node(entry.get())

        error_count = 0
        for name_node in entry.get().get_node_list(SERVICE_NAME_PATH):
            try:
                name = name_node.get_text()
                if self.__get_service_status(name) != RUNNING_STATUS:
                    continue

                self.__control_service(name, do_stop=True)
                self.__change_service_status(name, SUSPEND_STATUS)
            except:
                self.exception("Suspend error for " + name)
                error_count += 1

        if error_count:
            entry.set_result(Results.SUSPEND_ERROR)
        else:
            entry.set_result(Results.OK)

        return entry

    def __resume_handler(self, entry):
        """Handler for "resume" request."""

        self.__check_body_node(entry.get())

        error_count = 0
        try:
            target_list = [name_node.get_text() for name_node in \
                                entry.get().get_node_list(SERVICE_NAME_PATH)]
        except:
            target_list = []

        for suspend_service_path in glob.glob(SUSPEND_SERVICE_GLOB):
            try:
                name = os.path.basename(suspend_service_path)
                if len(target_list) and not name in target_list:
                    continue

                self.__control_service(name, do_stop=False)
                self.__change_service_status(name, RUNNING_STATUS)
            except:
                self.exception("Resume error for " + name)
                error_count += 1

        if error_count:
            entry.set_result(Results.RESUME_ERROR)
        else:
            entry.set_result(Results.OK)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.START, self.__start_handler)
        self.set_handler(ReqTypes.STOP, self.__stop_handler)
        self.set_handler(ReqTypes.SUSPEND, self.__suspend_handler)
        self.set_handler(ReqTypes.RESUME, self.__resume_handler)


#
# Main
#

if __name__ == "__main__":
    PalServiceCtrl(option=palcmd.PalCmdOption()).handle_request()
