#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for system log."""


import glob
import time
import re

import iobase.xmldata as xmldata
import libpal.palcmd as palcmd
from iobase.exception import NotFoundError
from pal_info import (ReqTypes, Results)
import pal_config as config

SYSLOG_MAX_FILE_NUM = 10
LOG_LINE_RE = re.compile(r'([0-9]+) .* .*audit: (.+)')
LOG_NODE = \
    "<log timestamp='%(timestamp)s' date='%(date)s'>" + \
    "<accessinfo>" + \
    "<username>%(username)s</username>" + \
    "<clientname>%(clientname)s</clientname>" + \
    "<client>%(client)s</client>" + \
    "<operation result='%(result)s'>%(operation)s</operation>" + \
    "<targetshare>%(targetshare)s</targetshare>" + \
    "<targetfile>%(targetfile)s</targetfile>" + \
    "</accessinfo>" + \
    "</log>"
OPTION_NAME = "option"
INDEX_KEY = "index"
MAXNUM_KEY = "maxnum"
SORT_KEY = "sort"
REVERSE_KEY = "reverse"
OPTION_XPATH = "./accesslog/option"
NONE_ATTR_VAL = "none"
TRUE_ATTR_VALUE = "true"


class PalSystemLog(palcmd.PalCmd):
    """Pal SystemLog implementation class."""

    def __is_exist_root_node(self, entry):
        """Check whether entry node has root node"""

        try:
            entry.get()
        except NotFoundError:
            return False
        return True

    def __is_exist_node(self, node, node_path):
        """Check whether node_path exists in specified node"""

        try:
            node.get_node(node_path)
        except NotFoundError:
            return False
        return True

    def __is_exist_attr(self, node, attr_val):
        """Check whether attr_val exists in specified node"""

        try:
            node.get_attr_value(attr_val)
        except NotFoundError:
            return False
        return True

    def __get_info_from_line(self, line):
        """Get the information form every log line."""

        dic = {
            "timestamp": "", "date": "",
            "username": "",
            "clientname": "", "client": "",
            "result": "",
            "operation": "",
            "targetshare": "", "targetfile": ""}
        matchobj = LOG_LINE_RE.match(line)

        dic["timestamp"] = matchobj.group(1)
        dic["date"] = time.strftime(
            "%Y/%m/%d %X", time.localtime(int(dic["timestamp"])))

        other_info_list = re.split('\|', matchobj.group(2))
        if other_info_list[0] == "$V2":
            # $V2|%U|%I|%m|%d|%S
            dic["username"] = other_info_list[1]
            dic["client"] = other_info_list[2]
            dic["clientname"] = other_info_list[3]
            dic["targetshare"] = other_info_list[5]
            dic["operation"] = other_info_list[6]
            dic["result"] = other_info_list[7]
        else:
            # %U|%I|%d|%S
            dic["username"] = other_info_list[0]
            dic["client"] = other_info_list[1]
            dic["targetshare"] = other_info_list[3]
            dic["operation"] = other_info_list[4]
            dic["result"] = other_info_list[5]

        last_item = other_info_list[len(other_info_list) - 1]
        if last_item != ".":
            dic["targetfile"] = last_item
        else:
            dic["targetfile"] = dic["targetshare"]

        if dic["operation"] == "open":
            rw_item = other_info_list[len(other_info_list) - 2]
            if rw_item == "w":
                dic["operation"] = "open_write"
            else:
                dic["operation"] = "open_read"

        return dic

    def __get_original_loglist_node(self):
        """Make a original loglist node that contains all log node."""

        loglist_node = xmldata.XmlNode("loglist")
        for audit_file in self.__get_audit_file_list():
            for line in open(audit_file, "r"):
                info_dic = self.__get_info_from_line(line)
                info_dic["targetfile"] = xmldata.escape(info_dic["targetfile"])
                log_node = xmldata.XmlConfNode(LOG_NODE % info_dic)
                loglist_node.append_child(log_node)
        return loglist_node

    def __get_audit_file_list(self):
        """Get a list of audit log file."""

        logfile_list = glob.glob(config.SMB_AUDIT_ROTATE_GLOB)
        logfile_list.sort()
        logfile_list.reverse()
        logfile_list.append(config.SMB_AUDIT_FILE)
        return logfile_list

    def __load_option_param(self, entry):
        """Load option node parameter"""

        option_node = entry.get().get_node(OPTION_XPATH)
        if self.__is_exist_attr(option_node, INDEX_KEY):
            self.__option_dict[INDEX_KEY] = \
                int(option_node.get_attr_value(INDEX_KEY))
        if self.__is_exist_attr(option_node, MAXNUM_KEY):
            self.__option_dict[MAXNUM_KEY] = \
                int(option_node.get_attr_value(MAXNUM_KEY))
        if self.__is_exist_attr(option_node, SORT_KEY):
            self.__option_dict[SORT_KEY] = \
                 option_node.get_attr_value(SORT_KEY)
        if self.__is_exist_attr(option_node, REVERSE_KEY):
            self.__option_dict[REVERSE_KEY] = \
                 option_node.get_attr_value(REVERSE_KEY)
        return

    def __get_specific_range_log_list(self, log_node_list):
        """Get specified range log list"""

        index = self.__option_dict[INDEX_KEY]
        maxnum = self.__option_dict[MAXNUM_KEY]

        if index is None and maxnum is None:
            return log_node_list
        if index is None:
            return log_node_list[:maxnum]
        if maxnum is None:
            return log_node_list[index:]
        if index >= 0:
            return log_node_list[index:index + maxnum]
        # index < 0
        #if len(log_node_list) < -1*index:
        #    return log_node_list[0: maxnum]
        if (index + maxnum) >= 0:
            return log_node_list[index:]

        return log_node_list[index: index + maxnum]

    def __sort_log(self, log_node_list):
        """Sort log_node_list"""

        if self.__option_dict[SORT_KEY] is None or \
            self.__option_dict[SORT_KEY] == NONE_ATTR_VAL:
            return log_node_list

        sort_target = \
            lambda node: node.get_attr_value(self.__option_dict[SORT_KEY])
        sorted_log_node_list = sorted(log_node_list, key=sort_target)

        return sorted_log_node_list

    def __reverse_log(self, log_node_list):
        """Reverse log_node_list"""

        if self.__option_dict[REVERSE_KEY] == TRUE_ATTR_VALUE:
            log_node_list.reverse()

        return log_node_list

    def __get_sorted_loglist_node(self):
        """Make a sorted loglist node."""

        log_node_list = []
        try:
            loglist_node = self.__get_original_loglist_node()
            log_node_list = loglist_node.get_node_list("log")
            self.__total_log_count += len(log_node_list)

            log_node_list = self.__sort_log(log_node_list)
            log_node_list = self.__reverse_log(log_node_list)
            log_node_list = self.__get_specific_range_log_list(log_node_list)
        except:
            pass

        new_loglist_node = xmldata.XmlNode("loglist")
        new_loglist_node.set_attrs({"total": str(self.__total_log_count)})
        try:
            for log_node in log_node_list:
                new_loglist_node.append_child(log_node)
        except:
            pass

        return new_loglist_node

    def __get_handler(self, entry):
        """Handle Get request and set entry to result."""

        if self.__is_exist_root_node(entry):
            if self.__is_exist_node(entry.get(), OPTION_XPATH):
                self.__load_option_param(entry)

        accesslog_node = xmldata.XmlNode("accesslog")
        accesslog_node.append_child(self.__get_sorted_loglist_node())
        entry.get().delete_child("accesslog")
        entry.get().append_child(accesslog_node)
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.__total_log_count = 0
        self.__option_dict = {
                    INDEX_KEY: None,
                    MAXNUM_KEY: None,
                    SORT_KEY: None,
                    REVERSE_KEY: None}

        self.set_handler(ReqTypes.GET, self.__get_handler)

#
# Main
#

if __name__ == "__main__":
    PalSystemLog(option=palcmd.PalCmdOption()).handle_request()
