#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for setting UPS."""


from __future__ import with_statement

import os
import re
import shutil
import glob
import time

import libpal.palcmd as palcmd
from pal_info import (ReqTypes, Results)
from iobase.exception import (InvalidArgError, NotFoundError, ExecError)
from iobase.cmdexec import (CmdExec, PIPE, NULL)
from ionas.confut import build_config_file
import pal_config as config

APC = "apc"
APC_FULL_NAME = "American Power Conversion"
OMRON = "omron"
FUJI = "fuji"
UPS = "ups"
STATUS_RE = "STATUS *: "
NO_BLANK_STATUS_RE = "STATUS *: \w+"
BATTERY_RE = "BCHARGE *: "
MODEL_RE = "^MODEL *: "
FIRMWARE_RE = "^FIRMWARE *: "
UPSCSCHED_CONF_TIMEOUT_RE = ":TIMEOUT:"
UPSC_STATUS_RE = "^([A-Z]+)\s+"
NUT_ONLINE_STATUS = "OL"
NUT_ONBATTERY_STATUS = "OB"
STATUS = "status"
BATTERY = "battery"
MODEL = "model"
FIRMWARE = "firmware"
WARNING = "warning"
VENDOR = "vendor"
NETSHUTDOWN = "netshutdown"
TARGETLANDISK_XPATH = "targetlandisklist/targetlandisk"
HOST = "host"
PASSWD = "passwd"

TIMEOUT_CONF = "TIMEOUT %s\n"

DEFAULT_TIMEOUT = "600"


class PalUps(palcmd.PalCmd):
    """Pal UPS command implementation class."""

    def __cmd_execute(self, cmd, stdout=NULL):
        """Execute the cmd and return the output of the cmd."""

        new_env = os.environ.copy()
        new_env['LANG'] = 'C'

        return CmdExec(cmd, stdout=stdout, stderr=NULL, env=new_env).execute()

    def __get_ups_infos(self):
        """Get the vendor of UPS."""

        infos = {VENDOR: ""}
        dev_path = ""
        for name in config.UPS_VENDOR_SUFFIX_LIST:
            path_list = glob.glob(config.DEV_DIR + "/*." + name)
            if len(path_list) > 0:
                infos[VENDOR] = name
                return (infos, path_list[0])
        return (infos, dev_path)

    def __get_ups_status(self):
        """Get the status of UPS, such as ups status, battery, etc."""

        (infos, dev_path) = self.__get_ups_infos()
        if infos[VENDOR] == APC:
            infos.update(self.__get_apc_status())
        elif infos[VENDOR] == OMRON:
            infos.update(self.__get_omron_status(dev_path))
        elif infos[VENDOR] == FUJI:
            infos.update(self.__get_fuji_status(dev_path))
        elif infos[VENDOR] == "":
            infos.update({STATUS: "DISCONNECT", BATTERY: ""})
        else:
            infos.update({STATUS: "UNKNOWN", BATTERY: ""})
        return infos

    def __get_omron_status(self, dev_path):
        """Get the infomation of OMRON UPS."""

        infos = {STATUS: "UNKNOWN", BATTERY: ""}
        cmd = [config.OMRONUPSCK_CMD, dev_path]
        proc = self.__cmd_execute(cmd, stdout=PIPE)
        status = proc.stdout.read()
        for line in status.split("\n"):
            if re.search(STATUS_RE, line):
                infos[STATUS] = re.sub(STATUS_RE, "", line)
            if re.search(BATTERY_RE, line):
                battery = re.sub(BATTERY_RE, "", line)
                infos[BATTERY] = str(float(battery))
        return infos

    def __get_apc_status(self):
        """Get the infomation of APC UPS."""

        infos = {STATUS: "UNKNOWN", BATTERY: ""}
        for num in range(1, 20):
            proc = self.__cmd_execute(config.APCACCESS_CMD, stdout=PIPE)
            status = proc.stdout.read()
            if proc.wait():
                time.sleep(1)
                continue

            if re.search(NO_BLANK_STATUS_RE, status):
                break
        for line in status.split("\n"):
            if re.search(STATUS_RE, line):
                status_sub = re.sub(STATUS_RE, "", line)
                if "ONBATT" in status_sub:
                    infos[STATUS] = "ONBATTERY"
                elif "ONLINE" in status_sub:
                    infos[STATUS] = "ONLINE"
                elif "COMMLOST" in status_sub:
                    infos[STATUS] = "COMMLOST"
                else:
                    infos[STATUS] = "UNKNOWN"
                continue
            if re.search(BATTERY_RE, line):
                battery_base = re.sub(BATTERY_RE, "", line)
                infos[BATTERY] = str(float(
                        re.sub(" Percent.*", "", battery_base)))
            if re.search(MODEL_RE, line):
                infos[MODEL] = re.sub(MODEL_RE, "", line)

            if re.search(FIRMWARE_RE, line):
                infos[FIRMWARE] = re.sub(FIRMWARE_RE, "", line)

        return infos

    def __get_fuji_status(self, dev_path):
        """Get the infomation of Fuji Electric UPS."""

        infos = {STATUS: "UNKNOWN", BATTERY: ""}
        try:
            cmd = [config.UPSC_CMD] + config.FUJIUPS_GET_CHARGE_ARGS
            proc = self.__cmd_execute(cmd, stdout=PIPE)
            infos[BATTERY] = str(float(proc.stdout.read().strip()))
        except:
            # ignore error
            pass

        try:
            cmd = [config.UPSC_CMD] + config.FUJIUPS_GET_STATUS_ARGS
            proc = self.__cmd_execute(cmd, stdout=PIPE)
            match = re.match(UPSC_STATUS_RE, proc.stdout.read().strip())
            if match != None:
                if match.group(1) == NUT_ONLINE_STATUS:
                    infos[STATUS] = "ONLINE"
                elif match.group(1) == NUT_ONBATTERY_STATUS:
                    infos[STATUS] = "ONBATTERY"
        except:
            # ignore error
            pass

        return infos

    def __set_apc_timeout(self, timeout):
        file = open(config.APC_TIME_PART_CONF, "w")
        file.write(TIMEOUT_CONF % timeout)
        file.close()

        self.__update_ups_conf()
        return

    def __update_ups_conf(self):
        (infos, dev_path) = self.__get_ups_infos()

        config_map = {
                ".apc": {
                        "": config.APC_USB_PART_CONF,
                        ".smart": config.APC_SMART_PART_CONF,
                        ".simple": config.APC_SIMPLE_PART_CONF,
                    },
            }
        (name_base, vendor) = os.path.splitext(dev_path)
        if vendor == ".apc":
            (name_base, type) = os.path.splitext(name_base)
            base = config_map[vendor][type]

            conf_file = os.path.join(config.APCUPSD_DIR, config.APC_CONF_FILE)
            build_config_file(conf_file, [base, config.APC_TIME_PART_CONF])
        return

    def __set_omron_timeout(self, timeout):
        tmp_file = config.OMRON_TMP_CONF + str(os.getpid())
        file = open(config.BASE_OMRON_CONF, "r")
        for line in file.readlines():
            line = re.sub(":TIMEOUT:", timeout, line)
            open(tmp_file, "a").write(line)
        if not os.path.exists(config.SSD_DIR):
            os.mkdir(config.SSD_DIR)
        conf_file = os.path.join(config.SSD_DIR, config.OMRON_CONF_FILE)
        shutil.move(tmp_file, conf_file)
        return

    def __set_fuji_timeout(self, timeout):
        tmp_file = config.NUT_UPSSCHED_TMP_CONF + str(os.getpid())
        with open(config.NUT_UPSSCHED_CONF_TMPL, "r") as tmpl:
            with open(tmp_file, "w") as tmp:
                tmp.write(re.sub(UPSCSCHED_CONF_TIMEOUT_RE,
                                    timeout, tmpl.read()))
        shutil.move(tmp_file, config.NUT_UPSSCHED_CONF)
        return

    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        ups_node = entry.get()
        if ups_node.get_node_name() != UPS:
            raise InvalidArgError(ups_node.get_node_name())

        timeout_min = int(ups_node.get_node("time/minute").get_text())
        timeout = str(timeout_min * 60)
        if timeout == "":
            timeout = DEFAULT_TIMEOUT
        self.__set_apc_timeout(timeout)
        self.__set_omron_timeout(timeout)
        self.__set_fuji_timeout(timeout)

        entry.set_result(Results.OK)
        return entry

    def __update_handler(self, entry):
        """Handle update request and set entry to result."""

        self.__update_ups_conf()
        entry.set_result(Results.OK)
        return entry

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        ups_node = entry.get()
        if ups_node.get_node_name() != UPS:
            raise InvalidArgError(ups_node.get_node_name())

        try:
            infos = self.__get_ups_status()
            for name, value in infos.items():
                ups_node.append_child(name, value)
        except:
            entry.set_result(Results.QUERY_ERROR)
            self.exception("Query Fail!")
            return entry

        entry.set_result(Results.OK)
        return entry

    def __netshutdown_handler(self, entry):
        """Handle networkshutdown request and set entry to result."""

        if entry.get().get_node_name() != NETSHUTDOWN:
            raise InvalidArgError(entry.get().get_node_name())

        if entry.get_type() == ReqTypes.CHECK:
            test_opt = ["-t"]
        else:
            test_opt = []

        target_list = entry.get().get_node_list(TARGETLANDISK_XPATH)
        for target in target_list:
            if not target.is_enabled():
                continue
            host = target.get_node(HOST).get_text()
            if host == "":
                continue
            passwd = target.get_node(PASSWD).get_text(do_strip=False)
            opts = ["-ip", host, "-passwd", passwd] + test_opt

            try:
                proc = self.__cmd_execute([config.NETSHUTDOWN_CMD] + opts)
                target.set_attrs({"shutdown": "success"})
            except ExecError, ins:
                if ins.get_returncode() == 2:
                    target.set_attrs({"shutdown": "auth_error"})
                elif ins.get_returncode() == 4:
                    target.set_attrs({"shutdown": "network_error"})
                else:
                    target.set_attrs({"shutdown": "other_error"})
            except:
                target.set_attrs({"shutdown": "other_error"})

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.NETSHUTDOWN, self.__netshutdown_handler)
        self.set_handler(ReqTypes.CHECK, self.__netshutdown_handler)
        self.set_handler(ReqTypes.UPDATE, self.__update_handler)

#
# Main
#

if __name__ == "__main__":
    PalUps(option=palcmd.PalCmdOption()).handle_request()
