#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for sending data to or receiving data from the URL."""


from __future__ import with_statement

import socket
import ssl

import urllib2
import urlparse
import StringIO

import iobase.netut as netut
from iobase.exception import (NotFoundError, InvalidArgError)

import libpal.palcmd as palcmd
from pal_info import (ReqTypes, Results)


URLACCESS_NAME = "urlaccess"
FILE_PROTOCOL = "file"
HTTPS_PROTOCOL = "https"
URL_NAME = "url"
SSL_CREATE_UNVERIFIED_CONTEXT = "_create_unverified_context"

SERVER_PATH = ".//proxy/serverlist/server"
DATA_NAME = "data"
HOST_NAME = "host"
PORT_NAME = "port"
PROXY_DIRECT = "DIRECT"

BUFFER_SIZE = 64 * 1024

SOCKET_TIMEOUT = 300

socket.setdefaulttimeout(SOCKET_TIMEOUT)


class PalUrlAccess(palcmd.PalCmd):
    """PAL URL access command implementation class."""

    def __check_body_node(self, node):
        """Check if entry's body node is valid."""

        if node.get_node_name() != URLACCESS_NAME:
            raise InvalidArgError(node.get_node_name())

        if not node.get_node(URL_NAME).get_text():
            raise InvalidArgError("URL empty")

    def __get_send_data(self, entry):
        """Return data for sending to URL."""

        try:
            data_file_name = entry.get_file()
            data = ""
            with open(data_file_name) as data_file:
                data = data_file.read()
        except:
            data = entry.get().get_node(DATA_NAME).get_text()
        return data

    def __get_server_str(self, protocol, host, port):
        """Return server string for a "host" and "port"."""

        server_str = host
        if port:
            if netut.is_inet6_addr(host):
                server_str = "[%s]:%s" % (host, port)
            else:
                server_str += ':' + port

        return urlparse.urlunparse((protocol, server_str, "", "", "", ""))

    def __get_proxy_server_list(self, entry, protocol):
        """Return proxy server list."""

        server_list = []
        try:
            for server in entry.get().get_node_list(SERVER_PATH):
                host = server.get_value(HOST_NAME)
                port = server.get_value(PORT_NAME)

                if not host or host == PROXY_DIRECT:
                    continue

                server_list.append(self.__get_server_str(protocol, host, port))
        except:
            pass

        return server_list

    def __get_url_handler_list(self, protocol, server):
        """Return URL handler list for urllib2.build_operner()."""

        handler_list = []
        if (protocol == HTTPS_PROTOCOL and
            hasattr(ssl, SSL_CREATE_UNVERIFIED_CONTEXT)):

            handler_list.append(urllib2.HTTPSHandler(
                                    context=ssl._create_unverified_context()))

        if protocol and server:
            handler_list.append(urllib2.ProxyHandler({protocol: server}))

        return handler_list

    def __url_open(self, entry, data=None):
        """Open URL specified "url" tag in "entry" and return its handle."""

        url = entry.get().get_node(URL_NAME).get_text()
        protocol = urlparse.urlparse(url)[0]

        try:
            server_list = self.__get_proxy_server_list(entry, protocol)
            if (not protocol or
                protocol == FILE_PROTOCOL or
                not server_list):

                if (protocol == HTTPS_PROTOCOL and
                    hasattr(ssl, SSL_CREATE_UNVERIFIED_CONTEXT)):

                    return urllib2.urlopen(url, data,
                                context=ssl._create_unverified_context())

                return urllib2.urlopen(url, data)

            for server in server_list:
                try:
                    opener = urllib2.build_opener(
                            *(self.__get_url_handler_list(protocol, server)))
                    return opener.open(url, data)
                except:
                    self.exception("urlopen for " + str(url) +
                                    " - proxy: " + str(server))

        except:
            self.exception("urlopen for " + str(url))

        return None

    def __send_handler(self, entry):
        """Handle send request and set entry to result."""

        self.__check_body_node(entry.get())

        url_obj = self.__url_open(entry, self.__get_send_data(entry))
        if not url_obj:
            entry.set_result(Results.SEND_ERROR)
        else:
            url_obj.close()
            entry.set_result(Results.OK)

        return entry

    def __copy_data(self, src_obj, dest_obj):
        """Read data from "src_obj" and write it to "dest_obj"."""

        while True:
            data = src_obj.read(BUFFER_SIZE)
            if not data:
                break

            dest_obj.write(data)

    def __receive_data(self, url_obj, entry):
        """Receive data from "url_obj" and save it to a file or "entry"."""

        try:
            receive_file = entry.get_altfile()

            with open(receive_file, "w") as data_file:
                self.__copy_data(url_obj, data_file)
        except NotFoundError:
            data_io = StringIO.StringIO()
            self.__copy_data(url_obj, data_io)
            data_io.seek(0)
            try:
                entry.get().get_node(DATA_NAME).set_text(data_io.read())
            except:
                entry.get().append_child(DATA_NAME, data_io.read())

    def __receive_handler(self, entry):
        """Handle receive request and set entry to result."""

        self.__check_body_node(entry.get())

        url_obj = self.__url_open(entry)
        if not url_obj:
            entry.set_result(Results.CONNECT_ERROR)
            return entry

        try:
            self.__receive_data(url_obj, entry)
            entry.set_result(Results.OK)
        except:
            self.exception("Recive Error")
            entry.set_result(Results.RECEIVE_ERROR)

        url_obj.close()
        return entry

    def __get_handler(self, entry):
        """Handle get request and set entry to result."""

        self.__check_body_node(entry.get())

        url_obj = self.__url_open(entry, self.__get_send_data(entry))
        if not url_obj:
            entry.set_result(Results.SEND_ERROR)
            return entry

        try:
            self.__receive_data(url_obj, entry)
            entry.set_result(Results.OK)
        except:
            self.exception("Recive Error")
            entry.set_result(Results.RECEIVE_ERROR)

        url_obj.close()
        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.SEND, self.__send_handler)
        self.set_handler(ReqTypes.RECEIVE, self.__receive_handler)
        self.set_handler(ReqTypes.GET, self.__get_handler)


#
# Main
#

if __name__ == "__main__":
    PalUrlAccess(option=palcmd.PalCmdOption()).handle_request()
