#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler to start or stop service for remotelink3 package."""

import os
import time

import ionas.request as request
import evh_config as config
import libevh.evhcmd as evhcmd
from fal_info import (FAL_SERVICECTRL_PATH, FAL_REMOTELINK3_PATH,
                        Results, ReqTypes)

from pkgaddon.remotelink3_config import (REMOTELINK3_SERVICE, PINCODE_PATH,)


EVENTINFO_NAME = "eventinfo"
VOLUME_NAME_PATH = "volume/name"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

SYSTEM_CATEGORY = "system"
INITIALIZE_ACTION = "initialize"

SERVICE_XPATH_TMPL = 'servicelist/service/name[text()="%s"]/..'
SERVICECTRL_REQ_XML_TMPL = """<servicectrl>
    <servicelist>
        <service><name>%s</name></service>
    </servicelist>
</servicectrl>
"""

REMOTELINK3_NAME = "remotelink3"
REMOTEACCESS_PATH = "remoteaccess"

DELETE_PINCODE_REQ = "delete_pincode"

SERVICE_WAIT_TIME = 2


class EvhRemoteLink3Initialize(evhcmd.EvhCmd):
    """Event handler to start or stop services for remotelink3 service."""

    def __send_request(self, target, type, body=""):
        """Send request entry to "target".""" 

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __service_control(self, name, type):
        """Control service."""

        self.__send_request(FAL_SERVICECTRL_PATH, type,
                        SERVICECTRL_REQ_XML_TMPL % (name,))

    def _is_service_enabled(self):
        """Check if service is enabled for remote link 3."""

        try:
            config = self.__send_request(FAL_REMOTELINK3_PATH, ReqTypes.LOAD,
                                REMOTELINK3_NAME).get()
            if config.get_node(REMOTEACCESS_PATH).is_enabled():
                return True
        except:
            pass
        return False

    def __handle_system_initialize_event(self, entry, is_previous):
        """Handle system:initialize event."""

        entry.set_result(Results.OK)
        if is_previous and os.path.exists(PINCODE_PATH):
            is_service_enabled = self._is_service_enabled()
            if not is_service_enabled:
                self.__service_control(REMOTELINK3_SERVICE, ReqTypes.START)
                time.sleep(SERVICE_WAIT_TIME)

            res_entry = self.__send_request(FAL_REMOTELINK3_PATH,
                                        DELETE_PINCODE_REQ, REMOTELINK3_NAME)
            entry.set_result(res_entry.get_result())

            if not is_service_enabled:
                self.__service_control(REMOTELINK3_SERVICE, ReqTypes.STOP)
        return entry

    def __handle_system_event(self, eventinfo, entry):
        """Handle system event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if action == INITIALIZE_ACTION:
            return self.__handle_system_initialize_event(entry, is_previous)

        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == SYSTEM_CATEGORY:
                return self.__handle_system_event(eventinfo, entry)

        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EvhRemoteLink3Service(option=evhcmd.EvhCmdOption()).handle_request()
