# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Remote Link 3 PIN code utility module.

Define functions for PIN code.
"""

import os
import urllib2
import urllib
import uuid
import socket
import re

from iobase.xmldata import (XmlConfNode,)


DATA_HEADER = "[RL3]\r\n"
PINCODE_DATA_FMT = "PINCODE=%s\r\n"
RL3USER_DATA_FMT = "RL3USER=%s\r\n"
RL3PASS_DATA_FMT = "RL3PASS=%s\r\n"
DEVNAME_DATA_FMT = "DEVNAME=%s\r\n"
QR_CODE_FMT = "[RL3]\r\nREV=4\r\nDATA=%s\r\n"

TIMEOUT = 30
REGISTRATION_PATH = "/InfoRegistration"
DELETION_PATH = "/InfoDeletion"

RESULT_XPATH = "result"

XML_ENCODING_RE = """<\?xml\s+.*\s+encoding\s*=\s*["']([^"']*)["'].*\?>\s*"""


def _to_data_str(pincode, user, password, dev_name):
    """Return stirng for DATA corresponding "pincode"."""

    data = DATA_HEADER
    data += PINCODE_DATA_FMT % (pincode,)

    if user != None:
        data += RL3USER_DATA_FMT % (user,)
    if password != None:
        data += RL3PASS_DATA_FMT % (password,)
    if dev_name != None:
        data += DEVNAME_DATA_FMT % (dev_name,)

    base64_data = data.encode("base64_codec").replace("\n", "")
    return base64_data.encode("rot_13")


def to_qrcode_str(pincode, user=None, password=None, dev_name=None):
    """Return string for QR code corresponding "pincode"."""

    return QR_CODE_FMT % (_to_data_str(pincode, user, password, dev_name),)


class RequestError(Exception):
    """Request Error class."""

    def __init__(self, result):
        """Constructor."""

        self.result = result

    def __str__(self):
        """Return the exception information as string."""

        return "PIN code Request Error: result: %s" % (str(self.result),)

    def get_result(self):
        """Return a result."""

        return self.result


class PinCode(object):
    """PinCode Class for managing a Remote Link 3 PIN code."""

    RESULT_OK = "00"
    RESULT_ALREADY_DELETED = "01"
    RESULT_ALREADY_REGISTERED = "02"

    def _send_to_center_server(self, cs_url, param):
        """Send "data" to center server."""

        old_timeout = socket.getdefaulttimeout()
        socket.setdefaulttimeout(self.timeout)

        response = urllib2.urlopen(cs_url, urllib.urlencode(param))

        socket.setdefaulttimeout(old_timeout)

        xml_str = response.read()
        # Work around xmldata module bug.
        encoding = None
        matchobj = re.search(XML_ENCODING_RE, xml_str)

        if matchobj != None:
            xml_str = re.sub(XML_ENCODING_RE, "", xml_str)
            encoding = str(matchobj.group(1)) 

        response_xml = XmlConfNode(xml_str, encoding=encoding)
        result = response_xml.get_value(RESULT_XPATH)
        if result != self.RESULT_OK:
            raise RequestError(result)

    def __init__(self, pincode=None, timeout=TIMEOUT):
        """Constructor."""

        self.pincode = pincode
        self.timeout = timeout

    def load(self, file_path):
        """Load a PIN code from "file_path"."""

        if os.path.exists(file_path):
            with open(file_path) as pincode_file:
                self.pincode = pincode_file.read().strip()
        else:
            self.pincode = None

    def save(self, file_path):
        """Save a PIN code to "file_path"."""

        if self.pincode == None:
            os.remove(file_path)
        else:
            with open(file_path, "w") as pincode_file:
                pincode_file.write(self.pincode)

    def set(self, pincode):
        """Set a PIN code."""

        self.pincode = pincode
        return self.pincode

    def get(self):
        """Return a PIN code."""

        return self.pincode

    def create(self, cs_base_url, encoded_key):
        """Create a new PIN code and register it to the center server."""

        self.pincode = str(uuid.uuid1()).replace("-", "").upper()
        param = {
                "enc_type": "2",
                "pin_code": self.pincode,
                "mac_address": encoded_key,
                "authority": "0",
            }

        self._send_to_center_server(cs_base_url + REGISTRATION_PATH, param)
        return self.pincode
        
    def delete(self, cs_base_url):
        """Delete a PIN code in the center server."""

        param = {
                "enc_type": "2",
                "pin_code": self.pincode,
            }

        self._send_to_center_server(cs_base_url + DELETION_PATH, param)
        self.pincode = None
